/*
* Copyright © 2021. TIBCO Software Inc.
* This file is subject to the license terms contained
* in the license file that is distributed with this file.
*/

// Application
app.controller('ApplicationCtrl', function($scope, ApplicationServices, TibcoCloudServices){
	var vm = this;	
	vm.ctrlName = 'ApplicationCtrl';

	const AUTH_OAUTH = "oauth";
	const AUTH_CREDENTIALS = "credentials";

	// Define login regions
	vm.regions = [
		/*{'name': 'AU', 'value': 'au'}, // No AU for now, no Spotfire */
		{'name': 'EU', 'value': 'eu'},
		{'name': 'US', 'value': 'us'}
	];

	// Application environment
	vm.environment = null;

	// User authentication/authorization
	vm.authenticated = null;
	vm.identification = null;
	vm.clientId = window.localStorage.getItem("clientId");
	vm.region = window.localStorage.getItem("region");
	vm.oauth = window.localStorage.getItem("oauth");
	//vm.authMode = vm.oauth != null ? AUTH_OAUTH : AUTH_CREDENTIALS;
	vm.authMode = AUTH_OAUTH;

	// Routing
	var initialRoute = "/home";
	var routesMap = {};
	vm.routes = [];
	vm.route = null;
	vm.routeTo = null;
	vm.routeNotFound = null;

	// Login properties
	vm.submitting = false;
	vm.credentials = null;
	vm.loginMessage = null;

	// --------------------------------------------------------------------------------
	// SERVICES

	// Load the environment from the server
	var loadEnvironment = function() {
		ApplicationServices.getEnvironment()
			.then(function(response){
				vm.environment = response;
				document.title = "TIBCO Cloud™ " + vm.environment.application.title;
			});
	};

	// Load the application configuration and route accordingly
	var loadApplicationConfiguration = async function() {
		return ApplicationServices.getApplicationConfiguration()
	};

	// Load routes configuration
	var loadRoutes = async function() {
		var promise = new Promise(function(resolve, reject){
			ApplicationServices.getRoutes()
			.then(function(response){
				// Set the routes and routeMap
				vm.routes = response.routes;
				for(var idx = 0; idx < response.routes.length; idx++)
					routesMap[response.routes[idx].routeId] = response.routes[idx];

				// Get initial route from URL if set
				var initialRouteUrl = new URLSearchParams(window.location.search).get("t");
				if(initialRouteUrl != null)
					initialRoute = initialRouteUrl;		
					
				// Navigate to the initial route
				vm.navigate(initialRoute);
				$scope.$digest();
				resolve();
			});
		});
		return promise;
	}
	
	// Test the specified id is a valid org id for the list of accounts
	var isOrgMember = function(id, accounts) {
		for(var thisAccount of accounts) {		
			if(thisAccount.accountId == id) {
				return true;
			}
		}
		return false;
	}

	// Identify the logged in user
	var identify = function() {
		// Identify returned success (but might not be authenticated)
		var success = async function(response) {
			console.log("Identify");
			console.log(response);
			
			// Set identification
			var authenticated = response.authenticated;
			vm.identification = response.identification;

			// Check for user/group permissions
			if(authenticated == true) {				
				// Load application configuration and routes
				vm.configuration = await loadApplicationConfiguration();				

				// Get the accounts
				vm.accounts = await getAccounts();
				console.log("Get Accounts");
				console.log(vm.accounts);

				// Check restriction
				if(vm.configuration.restriction != null && vm.configuration.restriction.type == 'owner' && 
						vm.configuration.restriction.id != vm.identification.hid) {
					authenticated = false;
					vm.loginMessage = "Unauthorized user";
				}
				else if(vm.configuration.restriction != null && vm.configuration.restriction.type == 'org' && 
						isOrgMember(vm.configuration.restriction.id, vm.accounts.accountsInfo) == false) {
					authenticated = false;
					vm.loginMessage = "Unauthorized group";
				}
				else {
					authenticated = true;
				}
			}
			else {
				authenticated = false;
			}

			// Set authenticated on controller
			vm.authenticated = authenticated;
			if(vm.authenticated == true) {
				// Set initial routes
				if(vm.configuration.showWelcomePage == true) {
					initialRoute = "/welcome";
				}

				// Load the routes, this will also navigate
				await loadRoutes();

				// Get the domain
				vm.domain = await getDomain();
				console.log("Get Domain");
				console.log(vm.domain);
				
			}

			$scope.$digest();
		};

		// Identify returned error
		var failed = function(err) {
			vm.authenticated = false;
			$scope.$digest();
		};

		// Call cloud services
		TibcoCloudServices.whoami(vm.region, vm.oauth)
			.then(success)
			.catch(failed);
	};

	// Login to TIBCO Cloud using credentials
	var login = function(username, password) {
		// Login returned success
		var success = function(response) {
			console.log("Login");
			console.log(response);

			vm.submitting = false;
			if(response.authenticated == true) {
				if(vm.authMode == AUTH_OAUTH)  {
					window.localStorage.removeItem('clientId');
					window.localStorage.setItem('region', vm.region);
					window.localStorage.setItem('oauth', vm.oauth);
				}
				else {
					window.localStorage.setItem('clientId', vm.clientId);
					window.localStorage.setItem('region', vm.region);
					window.localStorage.removeItem('oauth');
				}

				identify();
			}
			else {
				vm.loginMessage = "Invalid credentials or OAuth token"
			}

			$scope.$digest();
		};

		// Login returned failed
		var failed = function(xhr) {
			vm.submitting = false;
			if(xhr.message.responseJSON != null)
				vm.loginMessage = xhr.responseJSON.errorMsg;
			else
				vm.loginMessage = xhr;
			$scope.$digest();
		};

		// Set flags
		vm.submitting = true;
		vm.loginMessage = null;

		// Call identify based on authentication mode
		switch(vm.authMode) {
			case AUTH_OAUTH:
				TibcoCloudServices.whoami(vm.region, vm.oauth)
					.then(success)
					.catch(failed);
				break;
			default:
				TibcoCloudServices.loginCredentials(vm.region, username, password, 'tci', vm.clientId, vm.region)
					.then(success)
					.catch(failed);
		}
	};

	// Logout from TIBCO Cloud using credentials
	var logout = function() {
		window.localStorage.removeItem('oauth');
		TibcoCloudServices.logout()
			.then(function(){
				window.location.reload();
			});
	};

	// Retrieve TIBCO Cloud domain
	var getDomain = async function() {
		return TibcoCloudServices.getDomain(vm.region, vm.oauth);
	};

	// Retrieve TIBCO Cloud accounts
	var getAccounts = async function() {
		// Call cloud services
		return TibcoCloudServices.getAccounts(vm.region, vm.oauth);
	};

 	// --------------------------------------------------------------------------------
	// ACTIONS

	// Login
	vm.login = function(username, password) {
		if(vm.oauth != null)
			vm.authMode = AUTH_OAUTH;
		else
			vm.authMode = AUTH_CREDENTIALS;
		login(username, password);
	};

	// Logout
	vm.logout = function() {
		logout();
	};

	// UI Route
	vm.navigate = function(routeId) {
		$scope.$applyAsync(function() {
			vm.routeTo = routeId;
			var route = routesMap[routeId];
			if(route == null) {
				vm.routeNotFound = true;
			}
			else {
				vm.routeNotFound = false;
				vm.route = routesMap[routeId];
			}
		});
	};

	// --------------------------------------------------------------------------------
	// INITIALIZE

	loadEnvironment();
	identify();
});

// Header and Authentication
app.controller('HeaderCtrl', function(){	
	var vm = this;
	vm.ctrlName = 'HeaderCtrl';
});

// Welcome
app.controller('WelcomeCtrl', function($scope, ApplicationServices, ObjectHelper){
	var vm = this;
	var main = $scope.main;
	vm.ctrlName = 'WelcomeCtrl';

	// Set initial state
	vm.stateIdx = 0;
	vm.states = ['start', 'messaging', 'provision', 'settings', 'done'];

	// --------------------------------------------------------------------------------
	// SERVICES

 	// --------------------------------------------------------------------------------
	// ACTIONS

	// Call next state
	vm.changeState = async function(targetIdx) {
		if(targetIdx != null) {
			$scope.$applyAsync(function(){
				vm.stateIdx = targetIdx;
			});
		}
	};

	// Get next state
	vm.next = function() {
		if(vm.stateIdx == vm.states.length - 1)
			return null;
		else
			return vm.stateIdx + 1;
	};

	// Get previous state
	vm.prev = function() {
		if(vm.stateIdx == 0) 
			return null;
		else
			return vm.stateIdx - 1;
	};

	// Close the welcome page
	vm.close = function() {
		main.navigate('/home');
	}
});

// Provision
app.controller('ProvisionCtrl', function($scope, TibcoCloudServices, ApplicationServices){
	var vm = this;
	var main = $scope.main;
	vm.ctrlName = 'ProvisionCtrl';

	// Options
	vm.options = {
		provisionEvents: false,
		provisionDataStreams: false,
		oauth: main.oauth,
		tokenValid: null,
		tokenMessage: null,
		tcmChannel: main.configuration.messaging,
		sandboxes: null,
		sandboxId: null,
		state: "configure",
		executionMessage: null,
		executionError: false
	};

	// --------------------------------------------------------------------------------
	// SERVICES

	// Retrieve the Events sandboxes
	var getEventsSandboxes = async function() {
		var promise = new Promise(function(resolve, reject){
			TibcoCloudServices.getEventsSandboxes(main.region, vm.options.oauth)
				.then(function(response){
					vm.options.sandboxes = response;
					$scope.$digest();
					resolve();
				});
			});
		return promise;
	};

	// Execute provisioning process
	var provisionExecute = function() {
		vm.options.state = "execute";
		vm.options.executionMessage = null;

		var messageCallback = function(message) {
			var msg = JSON.parse(message);
			if(msg.code == 2)
				vm.options.executionError = true;
			vm.options.executionMessage = msg.message ? msg.message : msg.error;
			$scope.$digest();
		};

		// Compose payload
		var payload = {
			oauth: vm.options.oauth,
			region: main.region,
			provisionTypes: [],
			provisionTasks: null,
			sandboxId: vm.options.sandboxId,
			tcmChannelUrl: vm.options.tcmChannel.url,
			tcmChannelAuthKey: vm.options.tcmChannel.authKey
		}

		// Append types
		if(vm.options.provisionEvents)
			payload.provisionTypes.push("tce");
		if(vm.options.provisionDataStreams)
			payload.provisionTypes.push("tcds");

		// Call provision service
		ApplicationServices.provisionComponents(payload, messageCallback)
			.then(function(response){
				vm.options.state = "complete";
				$scope.$digest();
			});
	};

	// --------------------------------------------------------------------------------
	// ACTIONS

	// Validate OAuth
	vm.validateOAuth = function() {
		vm.options.tokenValue = false;
		vm.options.tokenMessage = null;
		if(vm.options.oauth == null || vm.options.oauth.length == 0) return;

		TibcoCloudServices.whoami(main.region, vm.options.oauth)
			.then(function(response){
				vm.options.tokenValid = response.authenticated;
				if(vm.options.tokenValid == false) {
					vm.options.tokenMessage = "Invalid OAuth Token";
				}
				else {
					vm.options.tokenMessage = "Valid OAuth Token";
				}
				$scope.$digest();
			});
	}
	
	// Determine if all provisioning requirements are met
	vm.canProvision = function() {
		if(vm.options.provisionEvents == false && vm.options.provisionDataStreams == false)		
			return false;
		if(vm.options.tokenValid == false || vm.options.tcmChannel == null || vm.options.tcmChannel.url == null || vm.options.tcmChannel.authKey == null)
			return false;
		//if(vm.options.provisionEvents == true && vm.options.sandboxId == null)
		//	return false;
		return true;
	};

	// Execute provisioning process
	vm.executeProvision = function() {
		provisionExecute();
	}

	// Load events sandboxes
	vm.loadEventsSandboxes = async function() {
		await getEventsSandboxes();
	};

	// --------------------------------------------------------------------------------
	// INITIALIZE
	vm.validateOAuth();
});

// Settings
app.controller('SettingsCtrl', function($scope){
	var vm = this;
	var main = $scope.main;
	vm.ctrlName = 'SettingsCtrl';

	// Categories
	vm.categories = [
		{"id": "application", "name": "Application", "icon": "settings", "showSave": true},
		{"id": "messaging", "name": "Messaging", "icon": "quickreply"},
		{"id": "provisioning", "name": "Provisioning", "icon": "account_tree"},
		{"id": "spotfire", "name": "Spotfire", "icon": "analytics", "showSave": true},
	];

	// Set initial selected category
	vm.selectedCategory = vm.categories[0];

	// --------------------------------------------------------------------------------
	// SERVICES

 	// --------------------------------------------------------------------------------
	// ACTIONS
	
	// Change category
	vm.changeCategory = function(category) {
		vm.selectedCategory = category;
	}	
});

// Application Settings
app.controller('ApplicationSettingsCtrl', function($scope, ApplicationServices, ObjectHelper){
	var vm = this;
	var main = $scope.main;
	var welcome = $scope.welcome;
	vm.ctrlName = 'ApplicationSettingsCtrl';

	// Options
	vm.options = {
		showWelcomePage: main.configuration.showWelcomePage,
		restriction: main.configuration.restriction == null ? "all" : main.configuration.restriction.type,
	}

	// --------------------------------------------------------------------------------
	// SERVICES

	// Process configuration options and save
	var processConfiguration = async function(options) {
		var configuration = ObjectHelper.cloneObject(main.configuration);

		// Welcome page
		configuration.showWelcomePage = options.showWelcomePage;		

		// Restriction
		if(options.restriction == 'all') {
			delete configuration.restriction;
		}
		else if(options.restriction == 'org') {
			configuration.restriction = {
				type: 'org',
				id: main.accounts.selectedAccount.accountId
			}
		}
		else if(options.restriction == 'owner') {
			configuration.restriction = {
				type: 'owner',
				id: main.identification.hid
			}
		}

		// Write the changes
		main.configuration = configuration;
		return ApplicationServices.saveApplicationConfiguration(configuration);		
	};

 	// --------------------------------------------------------------------------------
	// ACTIONS
	
	// Save configuration
	vm.save = async function(form) {
		await processConfiguration(vm.options);
		form.$setPristine();

		if(welcome != null) {
			welcome.changeState(welcome.next());
		}	
	}
});

// Messaging Settings
app.controller('MessagingSettingsCtrl', function($scope, ApplicationServices, ObjectHelper, TibcoCloudServices){
	var vm = this;
	var main = $scope.main;
	var welcome = $scope.welcome;
	vm.ctrlName = 'MessagingSettingsCtrl';

	// Flags
	vm.channelsLoading = null;

	// Options
	vm.options = {
		oauth: main.oauth,
		tokenValid: null,
		tokenMessage: null,
		tcmChannel: main.configuration.messaging,
		tcmChannels: null
	}

	// --------------------------------------------------------------------------------
	// SERVICES

	// Process configuration options and save
	var processConfiguration = async function(options) {
		var configuration = ObjectHelper.cloneObject(main.configuration);

		// Set messaging
		configuration.messaging = vm.options.tcmChannel;
		for(var key in configuration.messaging) {
			if(key.startsWith('$'))
				delete configuration.messaging[key];
		}

		// Write the changes
		main.configuration = configuration;
		return ApplicationServices.saveApplicationConfiguration(configuration);		
	};

	// Retrieve the TCM channels
	var getTcmChannels = function() {
		vm.channelsLoading = true;
		TibcoCloudServices.getTcmChannels(main.region, vm.options.oauth)
			.then(function(response){
				vm.options.tcmChannels = response.channels;
				vm.channelsLoading = false;
				$scope.$digest();
		});
	};

 	// --------------------------------------------------------------------------------
	// ACTIONS

	// Validate OAuth
	vm.validateOAuth = function() {
		vm.options.tokenValue = false;
		vm.options.tokenMessage = null;
		if(vm.options.oauth == null || vm.options.oauth.length == 0) return;

		TibcoCloudServices.whoami(main.region, vm.options.oauth)
			.then(function(response){
				vm.options.tokenValid = response.authenticated;
				if(vm.options.tokenValid == false) {
					vm.options.tokenMessage = "Invalid OAuth Token";
					vm.options.tcmChannels = null;
					vm.options.tcmChannel = null;
				}
				else {
					vm.options.tokenMessage = "Valid OAuth Token";
					getTcmChannels();
				}
				$scope.$digest();
			});
	}
	
	// Save configuration
	vm.save = async function(form) {
		await processConfiguration(vm.options);
		form.$setPristine();

		if(welcome != null) {
			welcome.changeState(welcome.next());
		}	
	}

	// --------------------------------------------------------------------------------
	// INITIALIZE
	vm.validateOAuth();
});

// Spotfire Settings
app.controller('SpotfireSettingsCtrl', function($scope, ApplicationServices, ObjectHelper, TibcoCloudServices){
	var vm = this;
	var main = $scope.main;
	var welcome = $scope.welcome;
	vm.ctrlName = 'SpotfireSettingsCtrl';

	// Flags
	vm.streamsLoading = null;

	// Options
	vm.options = {
		oauth: main.oauth,
		tokenValid: null,
		tokenMessage: null,
		streams: null,
		spotfire: {
			webPlayerServerRootUrl: main.configuration.spotfire.webPlayerServerRootUrl,
			analysisPath: main.configuration.spotfire.analysisPath
		}
	}

	// Set initial web player server based on region
	if(vm.options.spotfire.webPlayerServerRootUrl == null) {
		if(main.region == 'eu')
			vm.options.spotfire.webPlayerServerRootUrl = "https://eu.spotfire-next.cloud.tibco.com";
		else
			vm.options.spotfire.webPlayerServerRootUrl = "https://spotfire-next.cloud.tibco.com";
	}

	// --------------------------------------------------------------------------------
	// SERVICES

	// Process configuration options and save
	var processConfiguration = async function(options) {
		var configuration = ObjectHelper.cloneObject(main.configuration);

		// Set Spotfire
		configuration.spotfire.webPlayerServerRootUrl = vm.options.spotfire.webPlayerServerRootUrl;
		configuration.spotfire.analysisPath = vm.options.spotfire.analysisPath;

		// Write the changes
		main.configuration = configuration;
		return ApplicationServices.saveApplicationConfiguration(configuration);		
	};

	// Retrieve the data streams
	var getDataStreams = function() {
		vm.streamsLoading = true;
		TibcoCloudServices.getTcmChannels(main.region, vm.options.oauth)
			.then(function(response){
				vm.options.streams = response.streams;
				vm.streamsLoading = false;
				$scope.$digest();
		});
	};

	// Download the file
	var download = function() {
		let url = BASE_URI + '/service/spotfire/analysis/' + main.region;

		// Hacky method to download a file from a link with a token
		function downloadWithJwtViaFormPost(url, id, token) {
			var jwtInput = $('<input type="hidden" name="jwtToken">').val(token);
			var idInput = $('<input type="hidden" name="id">').val(id);
			$('<form method="get" target="_blank"></form>')
						.attr("action", url)
						.append(jwtInput)
						.append(idInput)
						.appendTo('body')
						.submit()
						.remove();
		}

		// Call the download
		downloadWithJwtViaFormPost(url, 'my-id', vm.options.oauth);
	}

 	// --------------------------------------------------------------------------------
	// ACTIONS

	// Validate OAuth
	vm.validateOAuth = function() {
		vm.options.tokenValue = false;
		vm.options.tokenMessage = null;
		if(vm.options.oauth == null || vm.options.oauth.length == 0) {
			vm.options.tokenValid = false;
			vm.options.tokenMessage = null;
			return;
		}

		TibcoCloudServices.whoami(main.region, vm.options.oauth)
			.then(function(response){
				vm.options.tokenValid = response.authenticated;
				if(vm.options.tokenValid == false) {
					vm.options.tokenMessage = "Invalid OAuth Token";
					vm.options.tcmChannels = null;
					vm.options.tcmChannel = null;
				}
				else {
					vm.options.tokenMessage = "Valid OAuth Token";
					getDataStreams();
				}
				$scope.$digest();
			});
	}
	
	// Save configuration
	vm.save = async function(form) {
		await processConfiguration(vm.options);
		form.$setPristine();

		if(welcome != null) {
			welcome.changeState(welcome.next());
		}	
	}

	// Download dxp
	vm.download = async function() {
		download();
	};

	// --------------------------------------------------------------------------------
	// INITIALIZE
	vm.validateOAuth();
});

// Home
app.controller('HomeCtrl', function($scope){
	var vm = this;
	var main = $scope.main;
	vm.ctrlName = 'HomeCtrl';
})

// Simulator
app.controller('SimulatorCtrl', function($scope, $log, ObjectHelper){
	var vm = this;
	var main = $scope.main;
	vm.ctrlName = 'SimulatorCtrl';

	// Initialize local variables
	var testcaseMap = {};

	// Initialize controller variables
	vm.testcases = [];
	vm.testcase = null;
	vm.compression = 3;

	// --------------------------------------------------------------------------------
	// SERVICES

	// Load test cases
	var loadTestCases = function() {
		var uri = BASE_URI + '/service/simulation/testcases';
		$.ajax({
			type: 'GET',
			url: uri,
			dataType: 'json',
			cache: false,
			success: function(response, textStatus, xhr) {
				vm.testcases = [];
				for(var idx = 0; idx < response.testcases.length; idx++) {
					var thisTestcase = response.testcases[idx].testcase;
					if(thisTestcase != undefined) {
						testcaseMap[thisTestcase.id] = thisTestcase;
						vm.testcases.push(thisTestcase);
					}
				}
				$scope.$digest();
			},
			error: function(xhr, textStatus, errorThrown) {
				console.error(xhr);
				$scope.$digest();
			}
		});					
	};
	
	// Start a new test case
	var startTestCase = function(testcase, compression) {
		// Compose the payload and the send object
		var request = {
			testcase: ObjectHelper.cloneObject(testcase),
			compression: compression
		};
		var jsonData = JSON.stringify(request, null, 2);

		var uri = BASE_URI + '/service/simulation/testcase/start';
		$.ajax({
			type: 'POST',
			url: uri,
			dataType: 'json',
			contentType: 'application/json',
			data: jsonData,
			cache: false,
			success: function(response, textStatus, xhr) {
				vm.testcase = null;
				main.navigate("/home");
				$scope.$digest();
			},
			error: function(xhr, textStatus, errorThrown) {
				console.error(xhr);
				$scope.$digest();
			}
		});			
	};

	// --------------------------------------------------------------------------------
	// ACTIONS
	vm.start = function() {
		startTestCase(vm.testcase, vm.compression);
	};

	// --------------------------------------------------------------------------------
	// STARTUP
	loadTestCases();
});
